/*
 * File:   LocalOsc - PIC24FJ16GA002 
 * Author: K.Wakizawa
 *
 * Created on June 13, 2024, 12:46 PM
 */

#pragma config IOL1WAY = OFF            // IOLOCK Protection (IOLOCK may be changed via unlocking seq)
#pragma config OSCIOFNC = ON            // Primary Oscillator Output Function (OSC2/CLKO/RC15 functions as port I/O (RC15))
#pragma config FNOSC = FRC              // Oscillator Select (Fast RC Oscillator (FRC))
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog Timer is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)

#include <xc.h>

#define LED LATBbits.LATB2
#define MUXO PORTBbits.RB3
#define LE LATBbits.LATB4
#define MOSI LATBbits.LATB5
#define SCK LATBbits.LATB6
#define CE LATBbits.LATB7

void
spi8( unsigned char d )
{
  int i;

  for( i = 0; i < 8; i ++ ) {
    if( 0x80 & d ) MOSI = 1;
    else           MOSI = 0;
    SCK = 1;
    d <<= 1;
    SCK = 0;
  }
}

void
pll_iniz( void )
{
  int i;

  LE = 0; // initialization latch
  spi8( 0b00110010 );
  //      00......  pre-scaler 8/9
  //      ...100..  Icp=3.13mA
  //      ......10  Icp=3.13mA
  spi8( 0b00111000 );
  //      0.......  Icp=3.13mA
  //      .0111...  PFD timeout 31cycles
  //      .....00.  High speed lock mode disable
  //      .......0  Charge pump normakl mode
  spi8( 0b10010011 );
  //      1.......  PD output positive
  //      .001....  MUXOUT -> digital lock detect
  //      ....0...  normal mode(power down disable)
  //      .....0..  normal mode(counter)
  //      ......11  Initalization latch  
  LE = 1;
  for( i = 0; i < 1000; i ++ );
  LE = 0; // function latch
  spi8( 0b00110010 );
  //      00......  pre-scaler 8/9
  //      ...100..  Icp=3.13mA
  //      ......10  Icp=3.13mA
  spi8( 0b00111000 );
  //      0.......  Icp=3.13mA
  //      .0111...  PFD timeout 31cycles
  //      .....00.  High speed lock mode disable
  //      .......0  Charge pump normakl mode
  spi8( 0b10010010 );
  //      1.......  PD output positive
  //      .001....  MUXOUT -> digital lock detect
  //      ....0...  normal mode(power down disable)
  //      .....0..  normal mode(counter)
  //      ......10  Function latch  
  LE = 1;
  for( i = 0; i < 1000; i ++ );
  LE = 0; // Reference counter
  spi8( 0b00000000 );
  //      .00.....  // noraml operation
  //      ...0....  // lock detect period
  //      ......00  // unti-back rush pulse width
  spi8( 0b00000000 ); // reference counter 1/50
  spi8( 0b11001000 );
  //      ......00  // Reference counter latch
  LE = 1;
  for( i = 0; i < 1000; i ++ );
  LE = 0; // AB counter
  spi8( 0b00000011 );
  //      ..0.....  // use charge pump setting #1
  //      ...00011  // B Counter
  spi8( 0b00111001 );
  //      ........  // B counter ->0x339 = 825
  spi8( 0b00000001 );
  LE = 1;
  for( i = 0; i < 1000; i ++ );

}

void
iniz( void )
{
  int i;
    
  TRISB = 0x0008; // RB3 is input, other pins are output
  LE = 1;
  SCK = 0;
  MOSI = 0;
  CE = 0;
  for( i = 0; i < 10000; i ++ ) ;
  CE = 1;
}

void
main( void ) 
{
  int i;
  
  iniz();
  pll_iniz();
  while( 1 ) {
    LED = 1;
    for( i = 0; i < 30000; i ++ ) ;
    LED = 0;
    for( i = 0; i < 30000; i ++ ) ;
  }
  return;
}
